window.Gleam.initSearch([{"doc":"act","title":"act","content":"# 🎬 act\n\n[![Package Version](https://img.shields.io/hexpm/v/act)](https://hex.pm/packages/act)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/act/)\n\nGleam is a functional programming language that does not support having _mutable state_. As such, programmers often have to pass state around manually, threading it through functions via arguments and return values. This can become a bit repetitive and clumsy.\n\nWhat if state could be &#39;threaded&#39; through functions automatically, with a nice API that resembles mutable state? This is the central idea of `act` and the [`Action`](https://hexdocs.pm/act/act#Action) type.\n\n```gleam\ntype Action(result, state) = fn(state) -&gt; #(state, result)\n```\n\n&gt; `act` is inspired by the (now outdated) [gleam-eval](https://github.com/hayleigh-dot-dev/gleam-eval) package. `gleam-eval` is super cool, but only supports the `Result` type and was not created with Gleam&#39;s `use` feature in mind.\n\n## Installation\n\n```sh\ngleam add act\n```\n\n## Docs &amp; Example\n\nDocumentation can be found at &lt;https://hexdocs.pm/act&gt;. A simple example lives in [test/act_test.gleam](https://github.com/MystPi/act/blob/main/test/act_test.gleam).\n\n## Development\n\n```sh\ngleam run   # Run the project\ngleam test  # Run the tests\n```\n","url":"index.html"},{"doc":"act","title":"Action","content":"pub type Action(result, state) =\n  fn(state) -&gt; #(state, result)\n An action is simply a function that takes some state and returns a value and\n a potentially updated state. Running an action is as simple as calling the\n function with a state.\n\n \n import gleam/int\n import act.{type Action}\n\n fn increment(by num: Int) -&gt; Action(String, Int) {\n   fn(state) {\n     #(state + num, &quot;I added &quot; &lt;&gt; int.to_string(by))\n   }\n }\n\n pub fn main() {\n   let initial_state = 0\n\n   initial_state\n   |&gt; act.all([increment(by: 2), increment(by: 5), increment(by: 1)])\n }\n\n // -&gt; #(8, [&quot;I added 2&quot;, &quot;I added 5&quot;, &quot;I added 1&quot;])\n \n\n As you can see, actions really are *just functions*! `act` simply provides a\n nice API for creating and working with these functions.\n\n\nSynonyms:\nact.Action\nact Action","url":"act.html#Action"},{"doc":"act","title":"ResultAction","content":"pub type ResultAction(ok, error, state) =\n  Action(Result(ok, error), state)\n An action that returns a `Result`, meaning it may fail.\n\n\nSynonyms:\nact.ResultAction\nact ResultAction","url":"act.html#ResultAction"},{"doc":"act","title":"all","content":"pub fn all(\n  actions: List(fn(a) -&gt; #(a, b)),\n) -&gt; fn(a) -&gt; #(a, List(b))\n Run a list of actions in sequence, returning a list of the results.\n\nSynonyms:\nact.all\nact all","url":"act.html#all"},{"doc":"act","title":"do","content":"pub fn do(\n  first_do: fn(a) -&gt; #(a, b),\n  and_then: fn(b) -&gt; fn(a) -&gt; #(a, c),\n) -&gt; fn(a) -&gt; #(a, c)\n Run the first action, passing its result to the `and_then` function which\n returns another action. This is very useful for chaining multiple actions\n together with `use` expressions.\n\n \n fn foo() {\n   use a_result &lt;- do(some_action)\n   use another_result &lt;- do(another_action(&quot;blah&quot;))\n   return(a_result &lt;&gt; another_result)\n }\n \n\n Using `use` is of course optional.\n\n \n fn bar() {\n   do(some_action, fn(result) {\n     io.debug(result)\n     return(result)\n   })\n }\n \n\nSynonyms:\nact.do\nact do","url":"act.html#do"},{"doc":"act","title":"each","content":"pub fn each(\n  actions: List(fn(a) -&gt; #(a, b)),\n) -&gt; fn(a) -&gt; #(a, Nil)\n Run a list of actions in sequence purely for updating state, ignoring their\n results. This function runs faster than `all` since it doesn&#39;t have to\n traverse the result list.\n\nSynonyms:\nact.each\nact each","url":"act.html#each"},{"doc":"act","title":"error","content":"pub fn error(value: a) -&gt; fn(b) -&gt; #(b, Result(c, a))\n Create an action that returns the given value wrapped in an `Error`.\n\nSynonyms:\nact.error\nact error","url":"act.html#error"},{"doc":"act","title":"get_state","content":"pub fn get_state() -&gt; fn(a) -&gt; #(a, a)\n Create an action that returns the current state. This is useful because\n functions such as `do` do not pass the updated state to their callbacks.\n\n \n fn foo() {\n   use original_state &lt;- do(get_state())\n   use result &lt;- do(some_action)\n   use new_state &lt;- do(get_state())\n   // do something with the variables\n }\n \n\nSynonyms:\nact.get_state\nact get_state","url":"act.html#get_state"},{"doc":"act","title":"map","content":"pub fn map(\n  action: fn(a) -&gt; #(a, b),\n  f: fn(b) -&gt; c,\n) -&gt; fn(a) -&gt; #(a, c)\n Transform the value produced by an action with the given function.\n\nSynonyms:\nact.map\nact map","url":"act.html#map"},{"doc":"act","title":"map_error","content":"pub fn map_error(\n  action: fn(a) -&gt; #(a, Result(b, c)),\n  f: fn(c) -&gt; d,\n) -&gt; fn(a) -&gt; #(a, Result(b, d))\n Transform the error produced by an action with the given function if it is\n wrapped in an `Error`, returning the `Ok` value otherwise.\n\nSynonyms:\nact.map_error\nact map_error","url":"act.html#map_error"},{"doc":"act","title":"map_ok","content":"pub fn map_ok(\n  action: fn(a) -&gt; #(a, Result(b, c)),\n  f: fn(b) -&gt; d,\n) -&gt; fn(a) -&gt; #(a, Result(d, c))\n Transform the value produced by an action with the given function if it is\n wrapped in an `Ok`, returning the `Error` otherwise.\n\nSynonyms:\nact.map_ok\nact map_ok","url":"act.html#map_ok"},{"doc":"act","title":"ok","content":"pub fn ok(value: a) -&gt; fn(b) -&gt; #(b, Result(a, c))\n Create an action that returns the given value wrapped in an `Ok`.\n\nSynonyms:\nact.ok\nact ok","url":"act.html#ok"},{"doc":"act","title":"return","content":"pub fn return(result: a) -&gt; fn(b) -&gt; #(b, a)\n Create an action that returns the given value and doesn&#39;t modify state.\n\n \n fn foo() -&gt; Action(String, s) {\n   use _ &lt;- do(update_something())\n   return(&quot;Updated!&quot;)\n }\n \n\nSynonyms:\nact.return\nact return","url":"act.html#return"},{"doc":"act","title":"set_state","content":"pub fn set_state(state: a) -&gt; fn(a) -&gt; #(a, Nil)\n Create an action that sets the current state to a new value, returning `Nil`.\n\n \n fn set_to_42() -&gt; Action(String, Int) {\n   use Nil &lt;- do(set_state(42))\n   return(&quot;The state is now 42! HAHAHAHA!!!&quot;)\n }\n \n\nSynonyms:\nact.set_state\nact set_state","url":"act.html#set_state"},{"doc":"act","title":"try","content":"pub fn try(\n  first_try: fn(a) -&gt; #(a, Result(b, c)),\n  and_then: fn(b) -&gt; fn(a) -&gt; #(a, Result(d, c)),\n) -&gt; fn(a) -&gt; #(a, Result(d, c))\n Like a combination of `do` and `result.try`. If the first action returns an\n `Ok` value, the `and_then` function is called with that value and the action\n that it returns is run. If the first action returns an `Error` value, the\n `and_then` function is not called and the error is returned.\n\nSynonyms:\nact.try\nact try","url":"act.html#try"},{"doc":"act","title":"try_all","content":"pub fn try_all(\n  actions: List(fn(a) -&gt; #(a, Result(b, c))),\n) -&gt; fn(a) -&gt; #(a, Result(List(b), c))\n Run a list of actions in sequence, stopping if an `Error` is encountered,\n and returning a list of the results.\n\nSynonyms:\nact.try_all\nact try_all","url":"act.html#try_all"},{"doc":"act","title":"try_each","content":"pub fn try_each(\n  actions: List(fn(a) -&gt; #(a, Result(b, c))),\n) -&gt; fn(a) -&gt; #(a, Result(Nil, c))\n Run a list of actions in sequence purely for updating state, stopping if an\n `Error` is encountered. This function runs faster than `try_all` since it\n doesn&#39;t have to traverse the result list.\n\nSynonyms:\nact.try_each\nact try_each","url":"act.html#try_each"},{"doc":"act","title":"update_state","content":"pub fn update_state(updater: fn(a) -&gt; a) -&gt; fn(a) -&gt; #(a, Nil)\n Create an action that updates the current state with the given function and\n returns `Nil`.\n\n \n fn increment_state(by: Int) -&gt; Action(Nil, Int) {\n   update_state(fn(s) { s + by })\n }\n \n\nSynonyms:\nact.update_state\nact update_state","url":"act.html#update_state"},{"doc":"act","title":"act","content":" Gleam is a functional programming language that does not support having\n *mutable state*. As such, programmers often have to pass state around manually,\n threading it through functions via arguments and return values. This can\n become a bit repetitive and clumsy.\n\n What if state could be &#39;threaded&#39; through functions automatically, with a\n nice API that resembles mutable state? This is the central idea of `act` and\n the [`Action`](#Action) type.\n","url":"act.html"}]);